/* 
 * File:   colorReduce.cpp
 * Author: gb
 *
 * Created on 4 de Novembro de 2015, 14:23
 */

#include <cstdlib>
#include "opencv2/opencv.hpp"

using namespace cv;

/*
 * Dithering usando Floyd–Steinberg
 * scale serve para indicar o tamanho do pixel
 */ 
void dithering(Mat &img, uchar scale=64){
    
    for (int r=1; r<img.rows-1; r++) {
        for (int c=1; c<img.cols-1; c++) {
            
            uchar oldPixel = img.at<uchar>(r,c); //captura pixel
            uchar newPixel = oldPixel / scale * scale;  //escala
            
            img.at<uchar>(r,c) = newPixel;  
            
            //e distribui a diferença nos vizinhos
            //dos quadrantes inferior direito
            int quantError = oldPixel - newPixel;  
            img.at<uchar>(r+1,c)   +=  7./16. * quantError;
            img.at<uchar>(r-1,c+1) +=  3./16. * quantError;
            img.at<uchar>(r,c+1)   +=  5./16. * quantError;
            img.at<uchar>(r+1,c+1) +=  1./16. * quantError;
        }
    }
}


int main(int, char**)
{
    VideoCapture cap(0); //abre a camera padrão. Estou num notebook!
    if(!cap.isOpened())  //checa se funcionou
        return -1;

    namedWindow("Dithering",1);  //define uma janela de apresentação
    for(;;)  //define um laço infinito para pegar vários frames
    {
        Mat frame;    
        cap >> frame; //obtem um novo frame a partir da camera
        cvtColor(frame,frame, COLOR_BGR2GRAY);
        dithering(frame);
        /*exibe o frame processado. Verifique que o primeiro
        parâmetro é equivalente ao nome da janela criada */
        imshow("Dithering", frame);  
        
        if(waitKey(30) >= 0) break;
    }
    return 0;
}
